#include "config.h"

#include "io.h"
#include "display.h"

#include <stdio.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#include <time.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

display d;

int displayInit(void)
{
	d.start_time = 0;
	d.total_time = 0;
	d.current_time = 0;
	d.elapsed_time = 0;
	d.percent_complete = 0.0;
	d.display_interval = 1;
	d.overtime_flag = 0;
	d.k = 1024;
	d.twiddle = '-';
	return(0);
}

int displayBegin(void)
{
	d.start_time = time(0);
	d.current_time = time(0);

	return(0);
}

#define sec_per_hour 3600
#define sec_per_minute 60

int calculateByteDisplay(unsigned long long *wptr,
	char **wXptr,
	float *wPptr
	)
{
	unsigned long long num = 0;
	unsigned long long div = 1;

	*wptr = io.total_write;
	*wXptr = "b ";
	*wPptr = 0.0;

	num = io.total_write;
	if (num >= d.k)         { *wXptr = "Kb"; div = d.k; }
	if (num >= (d.k * d.k)) { *wXptr = "Mb"; num /= d.k; }
	if (num >= (d.k * d.k)) { *wXptr = "Gb"; num /= d.k; }
	if (num >= (d.k * d.k)) { *wXptr = "Tb"; num /= d.k; }
	*wPptr = ((float)num / div);
	return(0);
}

int calculateBpsDisplay(unsigned long long *bpsptr,
	char **bpsXptr,
	float *bpsPptr
	)
{
	unsigned long long num = 0;
	unsigned long long div = 1;

	if (d.elapsed_time > 0) {
		*bpsptr = io.total_write / d.elapsed_time;
		num = io.total_write / d.elapsed_time;
	}
	else {
		*bpsptr = 0;
		num = 0;
	}
	*bpsXptr = "bps ";
	*bpsPptr = 0.0;

	if (num >= d.k)         { *bpsXptr = "Kbps"; div = d.k; }
	if (num >= (d.k * d.k)) { *bpsXptr = "Mbps"; num /= d.k; }
	if (num >= (d.k * d.k)) { *bpsXptr = "Gbps"; num /= d.k; }
	if (num >= (d.k * d.k)) { *bpsXptr = "Tbps"; num /= d.k; }
	*bpsPptr = ((float)num / div);
	return(0);
}

int calculateTimeDisplay(unsigned long long *sptr, unsigned long long *mptr, unsigned long long *hptr)
{
	if (*sptr > sec_per_hour) {
		*hptr = *sptr / sec_per_hour;
		*sptr -= *hptr * sec_per_hour;
	}
	if (*sptr > sec_per_minute) {
		*mptr = *sptr / sec_per_minute;
		*sptr -= *mptr * sec_per_minute;
	}
	return(0);
}

int calculatePercentComplete(void)
{
	if (io.total_size_known && (io.total_size > 0)) {
		d.percent_complete = (float)(io.total_write * 100.0 / io.total_size);
		if (d.percent_complete < 0) {
			d.percent_complete = 9999.9;
		}
	}
	return(0);
}

/*
char calculateTwiddle(void)
{
	clock_t n;
	char twiddle;

	n = abs(clock());
	switch ((n % CLOCKS_PER_SEC) / (CLOCKS_PER_SEC/4)) {
		case 0:  twiddle = '\\'; break;
		case 1:  twiddle = '|';  break;
		case 2:  twiddle = '/';  break;
		case 3:  twiddle = '-';  break;
	}
	return(twiddle);
}
*/

void displayTwiddle(void)
{
	static int t = 0;

	if (d.elapsed_time == 0) return;
	t += io.total_write / d.elapsed_time;
	if (t < 10) return;
	t = 0;
	switch (d.twiddle) {
		case '\\': d.twiddle = '|';  break;
		case '|':  d.twiddle = '/';  break;
		case '/':  d.twiddle = '-';  break;
		case '-':  d.twiddle = '\\'; break;
	}
	fprintf(stderr, "%c\r", d.twiddle);
}

int displayPrint(void)
{
	unsigned long long eta = 0;
	unsigned long long bps = 0;
	char *bpsX = "bps";
	float bpsP = 0.0;
	unsigned long long hours = 0;
	unsigned long long minutes = 0;
	unsigned long long seconds = 0;
	unsigned long long w = 0;
	char *wX = "";
	float wP = 0.0;
	char *time_title = "eta";
	int line_length = 31;
	int complete_length = 0;

	d.current_time = time(0);
	d.elapsed_time = d.current_time - d.start_time;

	if (calculateBpsDisplay(&bps, &bpsX, &bpsP) != 0) {
		return(1);
	}

	if (calculatePercentComplete() != 0) {
		return(1);
	}
	if (io.total_size_known && (io.total_size > 0)) {
		complete_length = line_length * d.percent_complete / 100;
	}

	if (io.total_size_known == 1) {
		if (io.total_write > 0) {
			if (io.total_size >= io.total_write) {
				if (d.percent_complete > 0.0) {
					eta = 
						(unsigned long long)(
							100 * d.elapsed_time / d.percent_complete
						)
						- d.elapsed_time;
				}
				else {
					eta = (unsigned long long)(-1);
				}
			}
			else {
				if (!d.overtime_flag) {
					d.overtime_flag = 1;
					d.total_time = d.elapsed_time;
				}
				eta = d.elapsed_time - d.total_time;
				time_title = "ovr";
			}
		}
		else {
			eta = 0;
		}
		seconds = eta;
	}
	else
	{
		seconds = d.elapsed_time;
		time_title = "elapsed";
	}
	if (calculateTimeDisplay(&seconds, &minutes, &hours) != 0) {
		return(1);
	}

	if (calculateByteDisplay(&w, &wX, &wP) != 0) {
		return(1);
	}

	fprintf(stderr, "%c ", d.twiddle);
		
	if (wP > 9999.9) {
		fprintf(stderr, "+999.9%2.2s at %6.1f%4.4s", wX, bpsP, bpsX);
	}
	else {
		fprintf(stderr, "%6.1f%2.2s at %6.1f%4.4s", wP, wX, bpsP, bpsX);
	}
	fprintf(stderr, "  %s: ", time_title);
	if (hours > 99) {
		fprintf(stderr, "+99:99:99");
	}
	else {
		fprintf(stderr, "%3llu:%2.2llu:%2.2llu", hours, minutes, seconds);
	}

	if (io.total_size_known) {
		int c;

		if (d.percent_complete > 999) {
			fprintf(stderr, " +999%% [");
		}
		else {
			fprintf(stderr, " %3d%% [", (int)d.percent_complete);
		}
		for (c = 0; c < line_length; c++) {
			if (c <= complete_length) {
				fprintf(stderr, "=");
			}
			else {
				fprintf(stderr, " ");
			}
		}
		fprintf(stderr, "]");
	}
	fprintf(stderr, "\r");

	return(0);
}

int displayUpdate(void)
{
	displayTwiddle();
	if (time(0) < d.current_time + d.display_interval) {
		fprintf(stderr, "\r");
		return(0);
	}
	if (displayPrint() != 0) return(1);
	return(0);
}

int displayEnd(void)
{
	unsigned long long bps = 0;
	char *bpsX = "bps";
	float bpsP = 0.0;
	unsigned long long w = 0;
	char *wX = "";
	float wP = 0.0;
	unsigned long long hours = 0;
	unsigned long long minutes = 0;
	unsigned long long seconds = 0;

	displayPrint();
	d.elapsed_time = d.current_time - d.start_time;

	if (calculatePercentComplete() != 0) {
		return(1);
	}

	if (calculateByteDisplay(&w, &wX, &wP) != 0) {
		return(1);
	}

	if (calculateBpsDisplay(&bps, &bpsX, &bpsP) != 0) {
		return(1);
	}

	seconds = d.elapsed_time;
	if (calculateTimeDisplay(&seconds, &minutes, &hours) != 0) {
		return(1);
	}

	/*
	fprintf(stderr, "\r                                                                               \r");
	*/
	fprintf(stderr, "\n");
	fprintf(stderr, "Bytes Copied: %llu (%.1f%s)", w, wP, wX);
	if (io.total_size_known) {
		fprintf(stderr, " (%d%% of expected input)", (int)d.percent_complete);
	}
	fprintf(stderr, "\n");

	fprintf(stderr, "Time: ");
	if (hours > 0) {
		fprintf(stderr, "%3llu:%2.2llu:%2.2llu", hours, minutes, seconds);
	}
	else if (minutes > 0) {
		fprintf(stderr, "%2.2llu:%2.2llu", minutes, seconds);
	}
	else {
		fprintf(stderr, "%2llu seconds", seconds);
	}
	fprintf(stderr, "\n");

	if ((hours != 0) || (minutes != 0) || (seconds != 0)) {
		fprintf(stderr, "Throughput: %llubps (%.1f%s)\n\n", bps, bpsP, bpsX);
	}
	else {
		fprintf(stderr, "Throughput: (infinite)\n\n");
	}

	d.start_time = 0;
	
	return(0);
}
