#include "config.h"

#include "fd.h"

#include <stdio.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include <sys/stat.h>
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif

/*
 * Return 1 if input is waiting on file descriptor, else return 0.
 */
/*
int fdIsReadReady(int fd)
{
	static struct timeval timeout = { 0, 0 };
	static fd_set rset;

	FD_ZERO(&rset);
	FD_SET(fd, &rset);
	select(fd+1, &rset, 0, 0, &timeout);
	if (FD_ISSET(fd, &rset)) return(1);
	return(0);
}
*/

/*
 * Return 1 if file descriptor is ready for output, else return 0.
 */
/*
int fdIsWriteReady(int fd)
{
	static struct timeval timeout = { 0, 0 };
	static fd_set wset;

	FD_ZERO(&wset);
	FD_SET(fd, &wset);
	select(fd+1, 0, &wset, 0, &timeout);
	if (FD_ISSET(fd, &wset)) return(1);
	return(0);
}
*/

int set_fl(int fd, int flags)
{
	int val;

	if ((val = fcntl(fd, F_GETFL, 0)) < 0) {
		fprintf(stderr, "*** ERROR: fcntl() failed\n");
		return(1);
	}
	val |= flags;
	if (fcntl(fd, F_SETFL, val) < 0) {
		fprintf(stderr, "*** ERROR: fcntl() failed\n");
		return(1);
	}
	return(0);
}

int clr_fl(int fd, int flags)
{
	int val;

	if ((val = fcntl(fd, F_GETFL, 0)) < 0) {
		fprintf(stderr, "*** ERROR: fcntl() failed\n");
		return(1);
	}
	val &= ~flags;
	if (fcntl(fd, F_SETFL, val) < 0) {
		fprintf(stderr, "*** ERROR: fcntl() failed\n");
		return(1);
	}
	return(0);
}

int fdBegin(int fd)
{
	return(set_fl(fd, O_NONBLOCK));
}

int fdEnd(int fd)
{
	return(clr_fl(fd, O_NONBLOCK));
}

int fdIsFile(int fd)
{
	struct stat st;
	if (fstat(fd, &st) != 0) {
		fprintf(stderr, "*** ERROR: fstat() failed\n");
		return(0);
	}
	if (S_ISREG(st.st_mode)) return(1);
	return(0);
}

int fdFileSize(int fd, unsigned long long *size)
{
	struct stat st;

	if (size == 0) {
		fprintf(stderr, "*** ERROR: size == NULL\n");
		return(1);
	}
	if (fstat(fd, &st) != 0) {
		fprintf(stderr, "*** ERROR: fstat() failed\n");
		return(1);
	}
	*size = (unsigned long long)st.st_size;
	return(0);
}

